"""Create classification report dataframe and show confusion matrix.

Abbreviation
=========== ========================================================
Shorthand    full name
=========== ========================================================
cr           classification report
cm           confusion matrix
=========== ========================================================
"""
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from quickshow.utils import find_all_files
from sklearn.metrics import classification_report
from sklearn.metrics import confusion_matrix


def vis_cm(
    df: pd.DataFrame,
    true_label_col: str,
    predicted_col: str,
    save_cr_csv_path: str,
    save_cmplot_path: str,
):
    """Create a heatmap using predicted column, ground truth label column.
    cr == classification report
    cm == confusion matrix

    Example code :
    df_cr, cm = vis_cm(df, 'real', 'predicted', './csv/exp1_cm.csv, './result/cm.png')
    vis_cm(df, 'ground_truth', 'predicted_y', None, None)

    Parameters
    ----------
    df : pd.DataFrame 
        pd.Dataframe object includes true label column and predicted_label column. 
    true_label_col : str
        column name which has ground_truth_labels.
    predicted_col : str
        column name which has predicted labels.
    save_cr_csv_path: str
        if you want to save df_cr, enter the full path. ('./result/df_cr.csv', or enter None)
    save_cmplot_path: str
        if you want to save cmplot image, enter the full path. ('./fig/cm.png' or enter None)
    """
    y_true = df[true_label_col]
    y_pred = df[predicted_col]
    classes = y_true.unique()

    try:
        cr = classification_report(
            y_true, y_pred, output_dict=True
        )  # create classification report
        df_cr = pd.DataFrame(cr)
    except Exception as e:
        print(e)
    try:
        cm = confusion_matrix(df[true_label_col], df[predicted_col])
    except Exception as e:
        print(e)

    plt.title("Confusion Metirx: True Label vs Predicted Label", fontsize=13, pad=20)
    sns.set_style("dark")
    ax = sns.heatmap(
        cm,
        cmap="crest",
        fmt=".3g",
        annot=True,
        xticklabels=classes,
        yticklabels=classes,
    )
    ax.set(xlabel="common xlabel", ylabel="common ylabel")
    plt.xlabel("Predicted", labelpad=10)
    plt.ylabel("True Label", labelpad=10)
    plt.show()
    if save_cmplot_path is not None:
        plt.savefig(save_cmplot_path, dpi=300, bbox_inches="tight")
    if save_cr_csv_path is not None:
        df_cr.to_csv(save_cr_csv_path, encoding="utf-8-sig", index=False)
    plt.clf()

    return df_cr, cm


def get_total_cr_df(cm_csv_folder_path: str, include_word: str, save_path: str) -> list:
    metric_df_list = find_all_files(cm_csv_folder_path, include_word)
    for i in range(len(metric_df_list)):
        if i == 0:
            total_cr_df = pd.read_csv(metric_df_list[i])
            exp_name = metric_df_list[i].split("\\")[-1].split("_")[0]
            total_cr_df["exp"] = exp_name
        else:
            exp_name = metric_df_list[i].split("\\")[-1].split("_")[0]
            df = pd.read_csv(metric_df_list[i])
            df["exp"] = exp_name
            total_cr_df = pd.concat([df, total_cr_df], ignore_index=True, axis=0)
    total_cr_df["exp"] = [int(x.lstrip("exp")) for x in total_cr_df["exp"]]

    if save_path is not None:
        total_cr_df.to_csv(save_path, index=False, encoding="utf-8-sig")
    total_cr_df.rename(columns={"Unnamed: 0": "metric"}, inplace=True)

    return total_cr_df


def vis_multi_plot(
    df: pd.DataFrame, which_metirc: str, except_col: str, save_path: str
) -> pd.DataFrame:
    df = df[df["metric"] == which_metirc]
    df = df.sort_values(by="exp", ascending=False)
    for i, col in enumerate(df.columns):
        if col not in except_col:
            plt.plot(df.exp, df[col], "-o", label=col, linewidth=1)
            plt.legend(loc="lower right")
        plt.grid(color="gray", linewidth=1)
        plt.legend
    plt.title(f"X: number of experiments, Y: {which_metirc}", fontsize=25, pad=40)
    plt.xlabel("Number of Experiments", fontsize=20, labelpad=30)
    plt.ylabel(f"{which_metirc} for each class", fontsize=20, labelpad=30)
    plt.show()
    if save_path is not None:
        plt.savefig(save_path, dpi=300, bbox_inches="tight")

    return df



"""Create and visualize dimensionally reduced pca and tsne columns through a specific column of clean dataframe.

Abbreviation
=========== ========================================================
Shorthand    full name
=========== ========================================================
t-SNE        t-distributed Stochastic Neighbor Embedding
PCA          Principal Component Analysis
=========== ========================================================
"""
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from sklearn.decomposition import PCA
from sklearn.manifold import TSNE
from matplotlib import get_backend
import warnings

warnings.filterwarnings("ignore")


def vis_tsne2d(
    df: pd.DataFrame,
    target_col: str,
    true_label_col: str,
    show_off: bool,
    save_plot_path: str,
) -> pd.DataFrame:
    """Create a 2d tsne columns through t-SNE(t-distributed Stochastic Neighbor Embedding) and simply visualize it.

    Example code:
    return_df = vis_tsne2d(embedded_df, 'target_col_embedding', 'category', True, './result/fig1.png')
    return_df = vis_tsne2d(input_df, 'token_array', 'bin_number', False, None)

    Parameters
    ----------
    df : pd.DataFrame 
        Data frame with target_col and true_label_col. 
    target_col : str
        Data column name to process pca included in data frame.
    true_label_col : str
        Label column name to be used for visualization if target_col has different labels.
        Enter None if label is missing or identical. (None or 'string of column name')
    show_off: bool
        If you want to hide plot when function called, use True. (True, False)
    save_plot_path: str
        Enter the full path to save result image.
    """
    data_subset = df[target_col].values
    raveled_data_subset = [x.ravel() for x in data_subset]

    tsne = TSNE(n_components=2)
    df_tsne_2d = tsne.fit_transform(raveled_data_subset)
    df["tsne-2d-1"] = df_tsne_2d[:, 0]
    df["tsne-2d-2"] = df_tsne_2d[:, 1]
    sns.scatterplot(
        x="tsne-2d-1",
        y="tsne-2d-2",
        hue=true_label_col,
        palette=sns.color_palette("Set2", len(df[true_label_col].unique())),
        data=df,
        legend="full",
    )
    if save_plot_path is not None:
        plt.title("T-SNE 3D")
        plt.savefig(save_plot_path, dpi=300, bbox_inches="tight")
    if show_off:
        plt.switch_backend("Agg")

    return df


def vis_tsne3d(
    df: pd.DataFrame,
    target_col: str,
    true_label_col: str,
    show_off: bool,
    save_plot_path: str,
) -> pd.DataFrame:
    """Create a 3d tsne columns through t-SNE(t-distributed Stochastic Neighbor Embedding) and simply visualize it.

    Example code:
    qs = QuickShow()
    return_df = qs.vis_tsne3d(embedded_df, 'target_col_embedding', 'category', True, './result/fig1.png')
    return_df = qs.vis_tsne3d(input_df, 'token_array', 'bin_number', False, None)
    
    Parameters
    ----------
    df : pd.DataFrame 
        Data frame with target_col and true_label_col. 
    target_col : str
        Data column name to process pca included in data frame.
    true_label_col : str
        Label column name to be used for visualization if target_col has different labels.
        Enter None if label is missing or identical. (`None` or `string of column name`)
    show_off: bool
        If you want to hide plot when function called, use True. (True, False)
    save_plot_path: str
        Enter the full path to save result image.
    """
    data_subset = df[target_col].values
    raveled_data_subset = [x.ravel() for x in data_subset]

    tsne = TSNE(n_components=3)
    df_tsne_3d = tsne.fit_transform(raveled_data_subset)
    df["tsne-3d-1"] = df_tsne_3d[:, 0]
    df["tsne-3d-2"] = df_tsne_3d[:, 1]
    df["tsne-3d-3"] = df_tsne_3d[:, 2]

    x, y, z = df["tsne-3d-1"], df["tsne-3d-2"], df["tsne-3d-3"]
    ax = plt.axes(projection="3d")
    ax.scatter3D(x, y, z)
    plt.title("T-SNE 3D")
    for s in df[true_label_col].unique():
        ax.scatter(
            df["tsne-3d-1"][df[true_label_col] == s],
            df["tsne-3d-2"][df[true_label_col] == s],
            df["tsne-3d-3"][df[true_label_col] == s],
            label=s,
        )
    ax.set_xlabel("tsne-3d-1")
    ax.set_ylabel("tsne-3d-2")
    ax.set_zlabel("tsne-3d-3")
    ax.legend()

    if save_plot_path is not None:
        plt.savefig(save_plot_path, dpi=300, bbox_inches="tight")
    if show_off:
        plt.switch_backend("Agg")

    return df


def vis_pca(
    df: pd.DataFrame,
    target_col: str,
    true_label_col: str,
    pca_dim: int,
    show_off: bool,
    save_plot_path: str,
) -> pd.DataFrame:
    """Create a pc columns through sklearn.decomposition and simply visualize it.

    Example code:
    return_df = qs.vis_pca(embedded_df, 'target_col_embedding', 'category', 2, True, './result/fig1.png')
    return_df = qs.vis_pca(input_df, 'token_array', 'bin_number', 3, False, None)

    Parameters
    ----------
    df : pd.DataFrame 
        Data frame with target_col and true_label_col. 
    target_col : str
        Data column name to process pca included in data frame.
    true_label_col : str
        Label column name to be used for visualization if target_col has different labels.
        Enter None if label is missing or identical. (`None` or `string of column name`)
    pca_dim : int
        Choose a dimension to visualize. (Enter 2 or 3 only)
    show_off: bool
        If you want to hide plot when function called, use True. (True, False)
    save_plot_path: str
        Enter the full path to save result image.

    Returns
    df : pd.DataFrame 
        Dataframe contains PC columns.
    -------
    """
    data_subset = df[target_col].values
    raveled_data_subset = [x.ravel() for x in data_subset]

    pca = PCA(n_components=3)
    pca_result = pca.fit_transform(raveled_data_subset)
    df["PC1"] = pca_result[:, 0]
    df["PC2"] = pca_result[:, 1]
    df["PC3"] = pca_result[:, 2]
    print(
        "Explained variation per PC(Principal Component): {}".format(
            pca.explained_variance_ratio_
        )
    )

    if pca_dim == 2:
        if true_label_col is None:
            ax.scatter(df["PC1"], df["pa-2"])
        else:
            sns.scatterplot(
                x="PC1",
                y="PC2",
                hue=true_label_col,
                palette=sns.color_palette("Set2", len(df[true_label_col].unique())),
                data=df,
                legend="full",
            )
            plt.title("PCA 2D")
            if save_plot_path is not None:
                plt.savefig(save_plot_path, dpi=300, bbox_inches="tight")

    elif pca_dim == 3:
        ax = plt.axes(projection="3d")
        if true_label_col is None:
            ax.scatter(df["PC1"], df["pa-2"], df["PC3"])
        else:
            for s in df[true_label_col].unique():
                ax.scatter(
                    df["PC1"][df[true_label_col] == s],
                    df["PC2"][df[true_label_col] == s],
                    df["PC3"][df[true_label_col] == s],
                    label=s,
                )
        ax.legend()
        ax.set_xlabel("PC1")
        ax.set_ylabel("PC2")
        ax.set_zlabel("PC3")
        if save_plot_path is not None:
            plt.title("PCA 3D")
            plt.savefig(save_plot_path, dpi=300, bbox_inches="tight")
    if show_off:
        plt.switch_backend("Agg")

    return df


def joint_tsne2d(
    df: pd.DataFrame,
    target_col: str,
    true_label_col: str,
    show_off: bool,
    save_plot_path: str,
) -> pd.DataFrame:
    """Create a 2d tsne columns through t-SNE(t-distributed Stochastic Neighbor Embedding) and simply visualize it.

    Example code:
    return_df = joint_tsne2d(df, 'values', 'labels', None, 'fig6.png')

    Parameters
    ----------
    df : pd.DataFrame 
        Data frame with target_col and true_label_col. 
    target_col : str
        Data column name to process pca included in data frame.
    true_label_col : str
        Label column name to be used for visualization if target_col has different labels.
        Enter None if label is missing or identical. (None or 'string of column name')
    show_off: bool
        If you want to hide plot when function called, use True. (True, False)
    save_plot_path: str
        Enter the full path to save result image.
    """
    data_subset = df[target_col].values
    raveled_data_subset = [x.ravel() for x in data_subset]

    tsne = TSNE(n_components=2)
    df_tsne_2d = tsne.fit_transform(raveled_data_subset)
    df["tsne-2d-1"] = df_tsne_2d[:, 0]
    df["tsne-2d-2"] = df_tsne_2d[:, 1]

    if true_label_col is None:
        sns.jointplot(df["tsne-2d-1"], df["tsne-2d-2"])
    else:
        sns.jointplot(
            x="tsne-2d-1",
            y="tsne-2d-2",
            hue=true_label_col,
            palette=sns.color_palette("Set2", len(df[true_label_col].unique())),
            data=df,
            legend="full",
        )
    if save_plot_path is not None:
        plt.title("T-SNE 2D")
        plt.savefig(save_plot_path, dpi=300, bbox_inches="tight")
    if show_off:
        plt.switch_backend("Agg")

    return df


def joint_pca2d(
    df: pd.DataFrame,
    target_col: str,
    true_label_col: str,
    show_off: bool,
    save_plot_path: str,
) -> pd.DataFrame:
    """Create a pc columns through sklearn.decomposition and simply visualize it and use seaborn.jointplot 

    Example code:
    return_df = qs.joint_pca2d(df, 'values', 'labels', None, 'fig5.png')

    Parameters
    ----------
    df : pd.DataFrame 
        Data frame with target_col and true_label_col. 
    target_col : str
        Data column name to process pca included in data frame.
    true_label_col : str
        Label column name to be used for visualization if target_col has different labels.
        Enter None if label is missing or identical. (`None` or `string of column name`)
    pca_dim : int
        Choose a dimension to visualize. (Enter 2 or 3 only)
    show_off: bool
        If you want to hide plot when function called, use True. (True, False)
    save_plot_path: str
        Enter the full path to save result image.

    Returns
    df : pd.DataFrame 
        Dataframe contains PC columns.
    -------
    """
    data_subset = df[target_col].values
    raveled_data_subset = [x.ravel() for x in data_subset]

    pca = PCA(n_components=2)
    pca_result = pca.fit_transform(raveled_data_subset)
    df["PC1"] = pca_result[:, 0]
    df["PC2"] = pca_result[:, 1]
    print(
        "Explained variation per PC(Principal Component): {}".format(
            pca.explained_variance_ratio_
        )
    )

    if true_label_col is None:
        sns.jointplot(df["PC1"], df["PC2"])
    else:
        sns.jointplot(
            x="PC1",
            y="PC2",
            hue=true_label_col,
            palette=sns.color_palette("Set2", len(df[true_label_col].unique())),
            data=df,
            legend="full",
        )
        plt.title("PCA 2D")
        if save_plot_path is not None:
            plt.savefig(save_plot_path, dpi=300, bbox_inches="tight")

    if show_off:
        plt.switch_backend("Agg")

    return df



import os


def find_all_files(folder_path: str, include_word: str) -> list:
    all_file_list = []
    for (root, directories, files) in os.walk(folder_path):
        for file in files:
            if include_word in file:
                file_path = os.path.join(root, file)
                all_file_list.append(file_path)

    return all_file_list


def rcparam(**args):
    """Some rcparam example
    import matplotlib.pyplot as plt
    plt.style.use('tableau-colorblind10')
    plt.rcParams["figure.figsize"] = (20,10)
    plt.rcParams['lines.linewidth'] = 4
    plt.rcParams['axes.grid'] = False 
    plt.rcParams['axes.facecolor'] = 'white'
    """



import seaborn as sns
import pandas as pd
import matplotlib.pyplot as plt

sns.set()


def vis_cluster_plot(
    df: pd.DataFrame, cluster_col: str, label_col: str, save_path: str
) -> pd.DataFrame:
    bin_list = [0]
    total = 0
    for i in range(16):
        count = len(
            df[df[[cluster_col, label_col]][cluster_col] == i][label_col].unique()
        )
        total += count
        bin_list.append(total - 1)
    df_pivot = df.pivot_table(
        index=[cluster_col, label_col], columns=label_col, aggfunc="size", fill_value=0
    )
    df_pivot.plot()

    plt.ylabel("Count of ground truth category", fontsize=20)
    plt.xlabel("Cluster bin", fontsize=20)
    plt.tick_params(axis="x", which="both", bottom=False, top=False, labelbottom=False)
    plt.title("how many real label is ditributed in each cluster", fontsize=20)
    plt.grid(True, axis="x", color="b", linewidth=1)
    plt.xticks(bin_list, label=False)
    if save_path is not None:
        plt.savefig(save_path, dpi=300, bbox_inches="tight")

    return df_pivot



# coding=utf-8
# Copyright 2023 parkminwoo Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# # limitations under the License.


from quickshow.reduce_dim import (
    vis_pca,
    vis_tsne2d,
    vis_tsne3d,
    joint_pca2d,
    joint_tsne2d,
)
from quickshow.cluster import vis_cluster_plot
from quickshow.eval_clf_model import vis_cm, get_total_cr_df, vis_multi_plot
from quickshow.utils import find_all_files

__all__ = [
    "vis_tsne2d",
    "vis_tsne3d",
    "vis_pca",
    "vis_cluster_plot",
    "vis_cm",
    "get_total_cr_df",
    "vis_multi_plot",
    "find_all_files",
    "rcparams",
    "joint_pca2d",
    "joint_tsne2d",
]

__version__ = "0.1.14"
__author__ = "MinWoo Park <parkminwoo1991@gmail.com>"



import re

from setuptools import find_packages
from setuptools import setup


def get_version():
    filename = "quickshow/__init__.py"
    with open(filename) as f:
        match = re.search(r"""^__version__ = ['"]([^'"]*)['"]""", f.read(), re.M)
    if not match:
        raise RuntimeError("{} doesn't contain __version__".format(filename))
    version = match.groups()[0]
    return version


def get_long_description():
    with open("README.md") as f:
        long_description = f.read()
        return long_description


version = get_version()


setup(
    name="quickshow",
    version="0.1.14",
    author="parkminwoo",
    author_email="parkminwoo1991@gmail.com",
    description="Quick-Show provides simply but powerful insight plots",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/DSDanielPark/quick-show",
    packages=find_packages(),
    python_requires=">=3.6",
    install_requires=["seaborn", "numpy", "pandas", "matplotlib", "scikit-learn"],
    keywords="Visuallization, Matplotlib, Plotting, Data Science",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
)
